@@ -1,5 +1,5 @@
 package IPC::Transit;
-$IPC::Transit::VERSION = '1.161300';
+$IPC::Transit::VERSION = '1.161400';
 use strict;use warnings;
 use 5.006;
 use IPC::Transit::Internal;
@@ -8,15 +8,25 @@ use Data::Dumper;
 use JSON;
 use Sereal::Encoder;
 use Sereal::Decoder qw(looks_like_sereal decode_sereal);
+use Sys::Hostname;
 use HTTP::Lite;
 use File::Temp qw/tempfile/;
 use Tie::DNS;
+use Crypt::Sodium;
+use MIME::Base64;
 
 use vars qw(
     $config_file $config_dir $large_transit_message_dir
     $local_queues
 );
 
+$IPC::Transit::my_keys = {
+    default => 'ftlMCefNymrF66r2VlFBgHYbWRZqSJPzVg4Vz/I86UQ='
+};
+$IPC::Transit::public_keys = {
+    default => 'vbqcxUUGIOvIKzpFWyBbYrSTsmSGj+/zlkF9H3tJ0DI='
+};
+
 our $large_transit_message_dir = '/tmp/transit_large_messages'
     unless $IPC::Transit::large_transit_message_dir;
 
@@ -32,6 +42,8 @@ our $wire_header_arg_translate = {
     message_length => 'l',
     local_filename => 'f',
     ttl => 't',
+    nonce => 'n',
+    source => 'S',
 };
 our $max_message_size = 4096 unless $IPC::Transit::max_message_size;
 
@@ -82,11 +94,15 @@ our $wire_header_args = {
     l => 1, #length of the message itself
     f => 1, #local_filename, optionally a path on the filesystem where the message can be found
     t => 1, #Time To Live
+    #for crypto
+    n => 1, #nonce
+    S => 1, #source
 };
 our $std_args = {
     message => 1,
     qname => 1,
     nowait => 1,
+    encrypt => 1,
 };
 
 sub send {
@@ -108,6 +124,9 @@ sub send {
         if ref $message ne 'HASH';
     $message->{'.ipc_transit_meta'} = {} unless $message->{'.ipc_transit_meta'};
     $message->{'.ipc_transit_meta'}->{send_ts} = time;
+    if($args{encrypt} and not $args{destination}) {
+        die "IPC::Transit::send: parameter 'destination' must exist if encryption is selected";
+    }
     if($args{destination}) {
         #let's take a stab at efficiently getting destination to either an
         #IP address or a FQDN
@@ -281,25 +300,6 @@ sub receive {
         if(not $from_queue->rcv($ref->{serialized_wire_data}, 102400000, 0, $flags)) {
             return undef;
         }
-        #legacy
-        if($ref->{serialized_wire_data} =~ /^file:(.*)/) {
-            my $filename = $1;
-            eval {
-                local $SIG{ALRM} = sub { die "timed out\n"; };
-                alarm 2;
-                open my $fh, '<', $filename
-                    or die "failed to open $filename for reading: $!";
-                read $fh, $ref->{serialized_wire_data}, 102400000
-                    or die "failed to read from $filename: $!";
-                close $fh or die "failed to close $filename: $!";
-            };
-            alarm 0;
-            unlink $filename;
-            if($@) {
-                print STDERR "legacy read from $filename failed: $@";
-                return undef;
-            }
-        }
         if(not defined $ref->{serialized_wire_data}) {
             print STDERR "IPC::Transit::receive: received message had no data";
             return undef;
@@ -321,12 +321,49 @@ sub receive {
             serialized_message => substr(
                 $ref->{serialized_wire_data},
                 $header_length + length($header_length) + 1,
-#                $wire_headers->{l}
-                9999999
+                9999999, # :(
             ),
         };
+        my $used_default_public = 1;
+        if($message->{wire_headers}->{n}) {
+            #we be encrypted
+            #validate $IPC::Transit::my_keys->{private}
+            #
+            my $source = $message->{wire_headers}->{S};
+            my $public_key;
+            if($IPC::Transit::public_keys->{$source}) {
+                $public_key = $IPC::Transit::public_keys->{$source};
+                $used_default_public = 0;
+            } else {
+                $public_key = $IPC::Transit::public_keys->{default};
+            }
+            my $private_key = $IPC::Transit::my_keys->{private};
+            $private_key = $IPC::Transit::my_keys->{default}
+                unless $private_key;
+            my $nonce = decode_base64($message->{wire_headers}->{n});
+            my $public_keys;
+            if(not ref $public_key) {
+                $public_keys = [$public_key];
+            } else {
+                $public_keys = $public_key;
+            }
+            my $cleartext;
+            foreach my $public (@$public_keys) {
+                $cleartext = crypto_box_open(
+                    $message->{serialized_message},
+                    $nonce,
+                    decode_base64($public),
+                    decode_base64($private_key),
+                );
+                last if $cleartext;
+            }
+            $message->{serialized_message} = $cleartext;
+        }
         return undef unless _thaw($message);
-
+        $message->{message}->{'.ipc_transit_meta'}->{encrypt_source} =
+            $message->{wire_headers}->{S} if $message->{wire_headers}->{S};
+        $message->{message}->{'.ipc_transit_meta'}->{encrypt_source} = 'default'
+            if $used_default_public;
         return $message if $args{raw};
         return $message->{message};
     };
@@ -432,7 +469,45 @@ sub pack_message {
         next if $std_args->{$key};
         $args->{message}->{'.ipc_transit_meta'}->{$key} = $args->{$key};
     }
-    $args->{serialized_message} = _freeze($args);
+    if($args->{encrypt}) {
+        $args->{message}->{'.ipc_transit_meta'}->{destination} = $args->{destination};
+    }
+    if($args->{encrypt}) {
+        my $sender = _get_my_hostname();
+        if(not $sender) {
+            die 'encrypt selected but unable to determine hostname.  Set $IPC::Transit::my_hostname to override';
+        }
+    }
+    if($args->{encrypt}) {
+        my $nonce = crypto_box_nonce();
+        $args->{nonce} = encode_base64($nonce);
+
+        my $my_private_key;
+        if($IPC::Transit::my_keys->{private}) {
+            $my_private_key = $IPC::Transit::my_keys->{private};
+            $args->{message}->{'.ipc_transit_meta'}->{signed_destination} = 'my_private';
+        } else {
+            $my_private_key = $IPC::Transit::my_keys->{default};
+            $args->{message}->{'.ipc_transit_meta'}->{signed_destination} = 'default';
+        }
+        my $their_public_key;
+        if($IPC::Transit::public_keys->{$args->{destination}}) {
+            $their_public_key = $IPC::Transit::public_keys->{$args->{destination}};
+        } else {
+            $their_public_key = $IPC::Transit::public_keys->{default};
+        }
+        $args->{serialized_message} = _freeze($args);
+        my $cipher_text = crypto_box(
+            $args->{serialized_message},
+            $nonce,
+            decode_base64($their_public_key),
+            decode_base64($my_private_key)
+        );
+        $args->{serialized_message} = $cipher_text;
+        $args->{source} = _get_my_hostname();
+    } else {
+        $args->{serialized_message} = _freeze($args);
+    }
     $args->{message_length} = length $args->{serialized_message};
     if($args->{message_length} > $IPC::Transit::max_message_size) {
         my $s;
@@ -542,6 +617,21 @@ sub _thaw {
     }
     return $ret;
 }
+
+sub gen_key_pair {
+    my ($pk, $sk) = box_keypair();
+    return (encode_base64($pk),encode_base64($sk));
+}
+
+{
+my $hostname;
+sub _get_my_hostname {
+    return $IPC::Transit::my_hostname if $IPC::Transit::my_hostname;
+    return $hostname if $hostname;
+    $hostname = hostname;
+    return $hostname;
+}
+}
 1;
 
 __END__
@@ -609,7 +699,7 @@ This queue framework has the following anti-goals:
 
 =head1 FUNCTIONS
 
-=head2 send(qname => 'some_queue', message => $hashref, serializer => 'some serializer')
+=head2 send(qname => 'some_queue', message => $hashref, [destination => $destination, serializer => 'some serializer', crypto => 1 ])
 
 This sends $hashref to 'some_queue'.  some_queue may be on the local
 box, or it may be in the same process space as the caller.
@@ -617,13 +707,20 @@ box, or it may be in the same process space as the caller.
 This call will block until the destination queue has enough space to
 handle the serialized message.
 
-The serialize_with argument is optional, and defaults to Data::Dumper.
-Currently, we are using the module Data::Serializer::Raw; any serialization
-scheme that module supports can be used here.
+The destination argument is optional.  If defined, it is the remote host
+will receive the message.
+
+The serialize argument is optional, and defaults to Sereal.  It is
+over-ridden with the IPC_TRANSIT_DEFAULT_SERIALIZER environmental
+variable.  The following serializers are available:
+
+serial, json, yaml, storable, dumper
 
 NB: there is no need to define the serialization type in receive.  It is
 automatically detected and utilized.
 
+The crypto argument is optional.  See below for details.
+
 =head2 receive(qname => 'some_queue', nonblock => [0|1], override_local => [0|1])
 
 This function fetches a hash reference from 'some_queue' and returns it.
@@ -660,15 +757,60 @@ Returns various stats about the passed queue name, per IPC::Msg::stat:
 Return an array of hash references, each containing the information 
 obtained by the stat() call, one entry for each queue on the system.
 
+=head2 CRYPTO
+
+On send(), if the crypto argument is set, IPC::Transit will sign and
+encrypt the message before it is sent.  The necessary configs, including
+relevant keys, are set in some global variables.
+
+See an actual example of this in action under ex/crypto.pl
+
+Please note that this module does not directly assist with the always
+onerous task of key distribution.
+
+=head3 $IPC::Transit::my_hostname
+
+If not set, this defaults to the output of the module Sys::Hostname.
+This value is placed into the message by the sender, and used by the
+receiver to lookup the public key of the sender.
+
+=head3 $IPC::Transit::my_keys
+
+This is a hash reference initially populated, in the attribute 'default',
+with the private half of a default key pair.  For actual secure
+communication, a new key pair must be generated on both sides, and the
+sender's private key needs to be placed here:
+
+  $IPC::Transit::my_keys->{private} = $real_private_key
+
+=head3 $IPC::Transit::public_keys
+
+As above, this is a hash reference initially populated, in the attribute
+'default', with the public half of a default key pair.  For actual secure
+communication, a new key pair must be generated on both sides, and the
+receiver's public key needs to be placed here:
+
+  $IPC::Transit::public_keys->{$receiver_hostname} = $real_public_key_from_receiver
+
+$receiver_hostname must exactly match what is passed into the 'destination'
+field of send().
+
+All of these keys must be base 64 encoded 32 byte primes, as used by
+the Crypto::Sodium package.
+
+=head3 IPC::Transit::gen_key_pair()
+
+This returns a two element array representing a public/privte key pair,
+properly base64 encoded for use in $IPC::Transit::my_keys and
+$IPC::Transit::public_keys
+
 =head1 SEE ALSO
 
 A zillion other queueing systems.
 
 =head1 TODO
 
-Crypto
-
-much else
+Implement nonblock flag for send()
 
 =head1 BUGS
 
@@ -676,7 +818,7 @@ Patches, flames, opinions, enhancement ideas are all welcome.
 
 I am not satisfied with not supporting Windows, but it is considered
 secondary.  I am open to the possibility of adding abstractions for this
-kind of support as long as it doesn't greatly affect the primary goals.
+kind of support as long as it doesn't impact the primary goals.
 
 =head1 COPYRIGHT
 
@@ -690,6 +832,6 @@ and/or modified under the terms of the Perl Artistic License
 
 =head1 AUTHOR
 
-Dana M. Diederich <diederich@gmail.com>
+Dana M. Diederich <dana@realms.org>
 
 =cut
